local tick = require 'tick'
function love.load()
    love.window.setFullscreen(true)
    width,height,flags=love.window.getMode()
    tick.framerate=30
    --math vars
    huge=math.huge
    pi=math.pi
    --graphics vars
    resolution={480,270}
    color={0,0,0}
    screen={r={},g={},b={}}
    for x=1,resolution[1],1 do
        screen.r[x]={}
        screen.g[x]={}
        screen.b[x]={}

        for y=1,resolution[2],1 do
            screen.r[x][y]=0
            screen.g[x][y]=0
            screen.b[x][y]=0
        end
    end

    --game vars
    test={}
    camera={x=50,y=0,z=50,angle=0}

    wall={x1={20},y1={20},z1={20},
          x2={-20},y2={20},z2={20},
          x3={-20},y3={-20},z3={20},
          x4={20},y4={-20},z4={20},}
end

function love.update(dt)
    if love.keyboard.isDown('escape') then
        love.window.minimize()
    end
    if love.keyboard.isDown('a') then
        camera.angle=camera.angle-2
        if camera.angle<0 then
            camera.angle=camera.angle+360
        end
    elseif love.keyboard.isDown('d') then
        camera.angle=camera.angle+2
        if camera.angle>359 then
            camera.angle=camera.angle-360
        end
    end
    if love.keyboard.isDown('w') then
        camera.x=camera.x+sin(toRad(camera.angle))
        camera.z=camera.z+cos(toRad(camera.angle))
    elseif love.keyboard.isDown('s')then
        camera.x=camera.x-sin(toRad(camera.angle))
        camera.z=camera.z-cos(toRad(camera.angle))
    end
end

function love.draw()
    for y=1,resolution[2],1 do
        for x=1,resolution[1],1 do
            screen.r[x][y]=0
            screen.g[x][y]=0
            screen.b[x][y]=0
        end
    end
    setColor(0,0.3,0.6)
    mx,my=getMouse()
    rectangle(0,0,resolution[1],resolution[2])
    setColor(1,1,0)
    drawWall(1)
    for y=1,resolution[2],1 do
        for x=1,resolution[1],1 do
            graphics.setColor(screen.r[x][y],screen.g[x][y],screen.b[x][y])
            graphics.rectangle('fill',(x-1)*(width/resolution[1]),(y-1)*(height/resolution[2]),(width/resolution[1]),(height/resolution[2]))
        end
    end
end

--graphics functions
graphics=love.graphics
function setColor(r,g,b)
    color[1],color[2],color[3]=r,g,b
end
function pixel(x,y)
    local xa=x
    if x==resolution[1] then
        xa=resolution[1]-1
    end
    screen.r[round(xa)+1][round(y)+1]=color[1]
    screen.g[round(xa)+1][round(y)+1]=color[2]
    screen.b[round(xa)+1][round(y)+1]=color[3]
end
function rectangle(x,y,sizey,sizex)
    for fy=0,sizex,1 do
        for fx=0,sizey,1 do
            local xa=fx+x
            if fx+x==resolution[1] then
                xa=resolution[1]-1
            end
            screen.r[round(xa)+1][round(fy+y)+1]=color[1]
            screen.g[round(xa)+1][round(fy+y)+1]=color[2]
            screen.b[round(xa)+1][round(fy+y)+1]=color[3]
        end
    end
end
function line(x1,y1,x2,y2)
    local angle=math.atan((x2-x1)/(y2-y1))
    local points={}
    for p=0,distance(x1,y1,x2,y2),0.1 do
        if y1<=y2 then
            pixel(sin(angle)*p+x1,cos(angle)*p+y1)
        else
            pixel(sin(angle+pi)*p+x1,cos(angle+pi)*p+y1)
        end
    end
end
function polygon(x1,y1,x2,y2,x3,y3,x4,y4)

end

--math functions
function abs(n)
    return math.abs(n)
end
function ceil(n)
    return math.ceil(n)
end
function cos(n)
    return math.cos(n)
end
function toDeg(n)
    return math.deg(n)
end
function distance(x1,y1,x2,y2)
    return math.sqrt((x1-x2)^2+(y1-y2)^2)
end
function floor(n)
    return math.floor(n)
end
function max(t)
    return math.max(unpack(t))
end
function min(t)
    return math.min(unpack(t))
end
function toRad(n)
    return math.rad(n)
end
function random(min,max)
    return math.random(min,max)
end
function round(n)
    return math.floor(n+0.5)
end
function sin(n)
    return math.sin(n)
end
function sqrt(n)
    return math.sqrt(n)
end
function tan(n)
    return math.tan(n)
end

function getMouse()
    local x,y=love.mouse.getPosition()
    x=x/(width/resolution[1])
    y=y/(height/resolution[2])
    return x,y
end

--game function
function test()
end

function project3d(x,y,z)
    local xRotate=(x-camera.x)*cos(toRad(camera.angle))-(z-camera.z)*sin(toRad(camera.angle))
    local zRotate=(x-camera.x)*sin(toRad(camera.angle))+(z-camera.z)*cos(toRad(camera.angle))
    return ((xRotate/zRotate)*300)+resolution[1]/2,((y/zRotate)*300)+resolution[2]/2
end
function drawWall(w)
    if wall.z1[w]<0 then
    --    x1=
    --    y1=
    --    z1=
    end
    local x1,y1=project3d(wall.x1[w],wall.y1[w],wall.z1[w])
    local x2,y2=project3d(wall.x2[w],wall.y2[w],wall.z2[w])
    local x3,y3=project3d(wall.x3[w],wall.y3[w],wall.z3[w])
    local x4,y4=project3d(wall.x4[w],wall.y4[w],wall.z4[w])
    if x1<resolution[1] and x1>0 and y1<resolution[2] and y1>0 and x2<resolution[1] and x2>0 and y2<resolution[2] and y1>0 then
        line(x1,y1,x2,y2)
    end
    if x2<resolution[1] and x2>0 and y2<resolution[2] and y2>0 and x3<resolution[1] and x3>0 and y3<resolution[2] and y3>0 then
        line(x2,y2,x3,y3)
    end
    if x3<resolution[1] and x3>0 and y3<resolution[2] and y3>0 and x4<resolution[1] and x4>0 and y4<resolution[2] and y4>0 then
        line(x3,y3,x4,y4)
    end
    if x4<resolution[1] and x4>0 and y4<resolution[2] and y4>0 and x1<resolution[1] and x1>0 and y1<resolution[2] and y1>0 then
        line(x4,y4,x1,y1)
    end
end